/***************************************************************************
                          kmatedit.cpp  -  description
                             -------------------
    begin                : Sam Mr 10 16:19:19 CET 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmatedit.h"
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qmultilineedit.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qcolor.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <kcolorbutton.h>


Kmatedit::Kmatedit(QWidget *parent, const char *name) : MainDlg(parent, name)
{
	// make lighttab enviroment ready
	prepareLight();
	TransformationExecutionOrderListView->setSorting(-1); // disable sorting for operation list
}

Kmatedit::~Kmatedit()
{
	if(lightsourceattribs != 0) delete lightsourceattribs;
}

/** generates the text output */
void Kmatedit::generateText()
{
	// first we check if the name of the scene object was given by the user...
	if(SceneObjEntry->text().isEmpty())
	{
		QMessageBox::warning(this,"kMatEdit","You have to enter the name of the\n desired scene object!",
								QMessageBox::Ok,QMessageBox::NoButton);
		SceneObjEntry->setFocus();
		return;
	}
	else
	{
		// so, clear output widget first
		MainOutput->clear();
		
		// now lets determine which tab was selected
		if(TabObjectType->currentPage() == lights) MainOutput->insertLine(makeLight());
		if(TabObjectType->currentPage() == camera) MainOutput->insertLine(makeCamera());
		if(TabObjectType->currentPage() == interface) MainOutput->insertLine(makeInterface());
		if(TabObjectType->currentPage() == transformation) MainOutput->insertLine(makeTransformBlock());
	}
	
}
/** generates the source code for a lightflow light source */
QString Kmatedit::makeLight()
{
	QString sequence = SceneObjEntry->text() + ".newLight(";
	QString tripel;
	
	// now insert light source type
	sequence = sequence + "\"" + LightType->currentText() + "\",[";
	// now lets fill the option block
	
	// all lightsources have at least the color parameter...
	sequence = sequence + "\"color\",vector3(";
	// now we have to code the rgb info...
	QColor color = LightColorEntry->color();
	int intensity = LightIntensityEntry->value();
	sequence = sequence + tripel.sprintf("%f,%f,%f)",color.red()*intensity / 255.0f,
																										color.green()*intensity / 255.0f,
																										color.blue()*intensity / 255.0f);
	
	// all other parameters depends on the light source type
	lightattrib_t lattribs = lightsourceattribs[(lightsource_t) LightType->currentItem()];
	
	// is the voulumetric pattern parameter required?
	if(lattribs & cpattern)
	{
		sequence = sequence + ",\"color\",\"";
		sequence = sequence + LightAmbientPatternEntry->text();
		sequence = sequence + "\"";
	}
	
  // is the patchpos parameter required ?
	if(lattribs & patch_pos)
    {
      sequence = sequence + ",\"position\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP1XEntry->text().toFloat(),
      																								 LightPatchP1YEntry->text().toFloat(),
      																								 LightPatchP1ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP2XEntry->text().toFloat(),
      																								 LightPatchP2YEntry->text().toFloat(),
      																								 LightPatchP2ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP3XEntry->text().toFloat(),
      																								 LightPatchP3YEntry->text().toFloat(),
      																								 LightPatchP3ZEntry->text().toFloat());
      sequence = sequence + ",vector3(";
			sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPatchP4XEntry->text().toFloat(),
      																								 LightPatchP4YEntry->text().toFloat(),
      																								 LightPatchP4ZEntry->text().toFloat());
    }
  // is the patch type parameter required?
  if(lattribs & patch_sides)
  {
  	sequence = sequence + ",\"type\",";
  	sequence = sequence + ((LightPatchTwoSidesCheck->isChecked()) ? "\"two-sided\"" :"\"one-sided\"");
  }
	// is the position parameter required?
  if(lattribs & position)
    {
      sequence = sequence + ",\"position\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightPosXEntry->text().toFloat(),
      																								 LightPosYEntry->text().toFloat(),
      																								 LightPosZEntry->text().toFloat());
    }
	// is the direction parameter required?
	if(lattribs & direction)
    {
      sequence = sequence + ",\"direction\",vector3(";
      sequence = sequence + tripel.sprintf("%f,%f,%f)",LightDirXEntry->text().toFloat(),
      																								 LightDirYEntry->text().toFloat(),
      																								 LightDirZEntry->text().toFloat());
    }
	// spotlight?
	if(lattribs & spotattribs)
		{
			sequence = sequence + ",\"angle\",";
			sequence = sequence + tripel.sprintf("%f",toRadians(LightAngleEntry->value()));
			
			sequence = sequence + tripel.sprintf(",%f",toRadians(LightPenumbraAngle->value()));
	
			sequence = sequence + ",\"exponent\",";
			sequence = sequence + tripel.sprintf("%f",LightPenumbraExponent->text().toFloat());
		}
	
	// is the decay value required and not equal to default = 2?
	if((lattribs & decay) && (LightDecayEntry->value() != 2))
	{
		sequence = sequence + ",\"decay\",";
		sequence = sequence + tripel.sprintf("%f",(float) LightDecayEntry->value());
	}
	
	// is the shadow flag required?
	if(lattribs & shadows)
	{
		sequence = sequence + ",\"shadows\",";
		sequence = sequence + ((LightShadowCheck->isChecked() == true) ? "1" : "0");
	}
	
	// soft light shadow parameters required?
	if(lattribs & softshadowattribs)
	{
		sequence = sequence + ",\"zbuffer\",";
		sequence = sequence + tripel.sprintf("%i",LightSoftZBuffer->value());
	
		sequence = sequence + ",\"radius\",";
		sequence = sequence + tripel.sprintf("%f",LightSoftRadius->value() / 100.0f);

		sequence = sequence + ",\"samples\",";
		sequence = sequence + tripel.sprintf("%i",LightSoftSamples->value());
	
		sequence = sequence + ",\"bias\",";
		sequence = sequence + tripel.sprintf("%f",LightSoftBias->text().toFloat());
	
	// finally, the file parameter if allowed
		if(LightSoftFileAttribCheck->isChecked())
		{
			sequence = sequence + ",\"file\",\"";
			sequence = sequence + LightSoftFileEntry->text();
			sequence = sequence + "\",\"";
			sequence = sequence + LightSoftFileFlagEntry->currentText();
			sequence = sequence + "\"";
		}
	}
	
	// terminating sequence
	sequence = sequence + "])";
	return sequence;
}

char *Kmatedit::imagesavers[] = {"tga-saver","saver"};

char *Kmatedit::lightsourceexp[] = {"ambient / background light",
																		 "sharp spot light",
																		 "for sunlight e.g.",
																		 "to simulate windows e.g.",
													           "to simulate neon fixtures e.g.",
													           "sharp point light",
													           "soft point lights",
													           "soft spot-lights",
													           "for soft sunlight e.g."};

													
/** disables all widgets in the light sheed 
	*	which are not required for the selected
	* light source type
	*/
void Kmatedit::lightDisableCheck(int index)
{
	lightattrib_t lattribs = lightsourceattribs[index];
	
	// now go to the map of widgets and enable / disable them
	typedef map<QWidget *,lightattrib_t>::const_iterator CI;
	
	for(CI p=widgetallow.begin(); p != widgetallow.end(); ++p)
	{
		// enable widget if a attribute which required it is set, otherwise disable it
		p->first->setEnabled(((p->second & lattribs) != 0));
	}
	
	if(LightSoftFileAttribCheck->isEnabled())
	{
		bool temp = LightSoftFileAttribCheck->isChecked();
		LightSoftFileEntry->setEnabled(temp);
		LightSoftFileFlagEntry->setEnabled(temp);
	}
	
	if(LightAmbientPatternCheck->isEnabled())
	{
		LightAmbientPatternEntry->setEnabled(LightAmbientPatternCheck->isChecked());
	}
}

/** initialise light tab enviroment */
void Kmatedit::prepareLight()
{
		// make an array for the 9 available lightsources
	lightsourceattribs = new lightattrib_t[9];
	
	// add all used attribues to the lightsource entries
	lightsourceattribs[ambient] = cpattern;
	lightsourceattribs[conic] = lightattrib_t( position | direction | shadows | spotattribs | decay );
	lightsourceattribs[directional] = lightattrib_t( direction | shadows );
	lightsourceattribs[directional_patch] = lightattrib_t( patch_pos | direction | shadows );
	lightsourceattribs[patch] = lightattrib_t( patch_pos | patch_sides | decay | shadows );
	lightsourceattribs[point] = lightattrib_t( position | decay | shadows );
	lightsourceattribs[soft] = lightattrib_t( position | decay | softshadowattribs );
	lightsourceattribs[soft_conic] = lightattrib_t( position | direction | spotattribs | decay | softshadowattribs );
	lightsourceattribs[soft_directional] = lightattrib_t( direction | softshadowattribs );
	
	// create a map of widgets which are not always used to the attributes which requires them
	widgetallow[LightShadowCheck] = shadows;
	widgetallow[LightPosXEntry] = position;
	widgetallow[LightPosYEntry] = position;
	widgetallow[LightPosZEntry] = position;
	widgetallow[LightDirXEntry] = direction;
	widgetallow[LightDirYEntry] = direction;
	widgetallow[LightDirZEntry] = direction;
	widgetallow[LightPatchP1XEntry] = patch_pos;
	widgetallow[LightPatchP1YEntry] = patch_pos;
	widgetallow[LightPatchP1ZEntry] = patch_pos;
	widgetallow[LightPatchP2XEntry] = patch_pos;
	widgetallow[LightPatchP2YEntry] = patch_pos;
	widgetallow[LightPatchP2ZEntry] = patch_pos;
	widgetallow[LightPatchP3XEntry] = patch_pos;
	widgetallow[LightPatchP3YEntry] = patch_pos;
	widgetallow[LightPatchP3ZEntry] = patch_pos;
	widgetallow[LightPatchP4XEntry] = patch_pos;
	widgetallow[LightPatchP4YEntry] = patch_pos;
	widgetallow[LightPatchP4ZEntry] = patch_pos;
	widgetallow[LightPatchTwoSidesCheck] = patch_sides;
	widgetallow[LightAngleEntry] = spotattribs;
	widgetallow[LightPenumbraAngle] = spotattribs;
	widgetallow[LightPenumbraExponent] = spotattribs;
	widgetallow[LightDecayEntry] = decay;
	widgetallow[LightSoftBias] = softshadowattribs;
	widgetallow[LightSoftFileAttribCheck] = softshadowattribs;
	widgetallow[LightSoftFileEntry] = softshadowattribs;
	widgetallow[LightSoftFileFlagEntry] = softshadowattribs;
	widgetallow[LightSoftZBuffer] = softshadowattribs;
	widgetallow[LightSoftSamples] = softshadowattribs;
	widgetallow[LightSoftRadius] = softshadowattribs;
	widgetallow[LightAmbientPatternCheck] = cpattern;
	widgetallow[LightAmbientPatternEntry] = cpattern;
	
	// initiate check
	lightDisableCheck(0);	
}
/** generates the source code for a lightflow camera & saver enviroment */
QString Kmatedit::makeCamera()
{
	QString sequence;
	QString numbers;
	
	// filter imager required?
	if(CameraFilterEnableCheck->isChecked())
	{
		sfxfilter_t filter = (sfxfilter_t) CameraFilterSelTab->currentPageIndex();
		if(CameraAddFilmCheck->isChecked() && (filter != film))
		{
			// user wishes additional film effect filter
			sequence = "prefilter = " + makeCameraFilter(filter) + "\n";
			// make film filter as sfxfilter
			sequence = sequence + "sfxfilter  = " + SceneObjEntry->text() + ".newImager(\"film\",[\"imager\",prefilter,\"grain\",";
			sequence = sequence + numbers.sprintf("%f])\n\n",CameraFilmGrainEntry->text().toFloat());
		}
		else
		{
			// no additional film filter or film filter selected twice (filter + addition)
		 	sequence = "sfxfilter = " + makeCameraFilter(filter) + "\n\n";
		}
	}
	
	// o.k. make the saver
	sequence = sequence + "saver = " + SceneObjEntry->text();
	sequence = sequence +  + ".newImager(\"";
	sequence = sequence + imagesavers[CameraTypeCombo->currentItem()];
	sequence = sequence + "\",[\"file\",\"";
	sequence = sequence + CameraFileEntry->text();
	sequence = sequence + CameraTypeCombo->currentText();
	sequence = sequence + "\"";
	
	if(CameraFilterEnableCheck->isChecked())
	{
		sequence = sequence + ",\"imager\",sfxfilter";
	}
	
	if(CameraAlphaCheck->isEnabled())
	{
		sequence = sequence + ",\"alpha\",";
		sequence = sequence + ((CameraAlphaCheck->isChecked()) ? "1" : "0");
	}
	sequence = sequence + "])\n\n";
	
	// now comes the camera
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".imagerBegin(saver)\n";
	// generate camera
	sequence = sequence + "camera = " + SceneObjEntry->text();
	sequence = sequence + ".newCamera(\"pinhole\",[\"eye\",";
	sequence = sequence + "vector3(" + CameraPosX->text() + "," + CameraPosY->text() + "," + CameraPosZ->text() + ")";
	sequence = sequence + ",\"aim\",vector3(" + CameraAimtoX->text() + "," + CameraAimtoY->text() + "," + CameraAimtoZ->text() + ")";
	// include aa parameters if desired
	if(CameraAASamplesCheck->isChecked())
	{
		sequence = sequence + ",\"aa-samples\"," + CameraAAareaSpin_2->text() + "," + CameraAAsupersamplingSpin->text();
	}
	if(CameraAAThresholdCheck->isChecked())
	{
	 	sequence = sequence + ",\"aa-threshold\"," + numbers.sprintf("%f",CameraAAThresholdEntry->text().toFloat());
	}
	if(CameraAAJitterCheck->isChecked())
	{
		sequence = sequence + ",\"aa-jitter\"," + numbers.sprintf("%f",CameraAAJitterEntry->text().toFloat());
	}
	// include extended view parameters if desired
	if(CameraViewUpCheck->isChecked())
	{
		sequence = sequence + ",\"up\",vector3(" + CameraViewUpX->text() + "," + CameraViewUpY->text() + "," + CameraViewUpZ->text() + ")";
	}
	if(CameraViewFovCheck->isChecked())
	{
		sequence = sequence + ",\"fov\"," + numbers.sprintf("%f",CameraViewVertEntry->text().toFloat());
		if(CameraViewFovHorizEntry->edited())
		{
			sequence = sequence + ",\"fov\",\"horizontal\"," + numbers.sprintf("%f",CameraViewFovHorizEntry->text().toFloat());
		}
	}
	if(CameraViewApertureCheck->isChecked())
	{
		sequence = sequence + ",\"aperture\"," + numbers.sprintf("%f",CameraViewApertureEntry->text().toFloat());
	}
	if(CameraViewAspectCheck->isChecked())
	{
		sequence = sequence + ",\"aspect\"," + numbers.sprintf("%f",CameraViewAspectEntry->text().toFloat());
	}
	sequence = sequence + "])\n";
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".imagerEnd(saver)\n\n";
	
	// enable radiosity?
	if(CameraRadiosityCheck->isChecked())
	{
		sequence = sequence + SceneObjEntry->text();
		sequence = sequence + ".radiosity()\n\n";
	}
	
	// give the render command
	sequence = sequence + SceneObjEntry->text();
	sequence = sequence + ".render(camera,";
	sequence = sequence + CameraResXEntry->text() + "," + CameraResYEntry->text();
	sequence = sequence + ")\n";
	return(sequence);
}

/**
	* disables / enables widgets dependent on the selected
	* widget type
	*/
	
void Kmatedit::cameraDisableCheck(int selection)
{
	// enable alpha channel option only for tga saver
	CameraAlphaCheck->setEnabled(selection == 0);
}

/**
	* generates the code for an imager filter
	* @param filter index of activated filter tab page
	*/
QString Kmatedit::makeCameraFilter(sfxfilter_t filter)
{
	QString numbers;
	QString sequence = SceneObjEntry->text() + ".newImager(\"";
	sequence = sequence + CameraFilterSelTab->tabLabel(CameraFilterSelTab->currentPage());
	sequence = sequence + "\",[";
	switch(filter)
	{
		case dof :
			sequence = sequence + "\"depth\"," + ((CameraDofAutofocusCheck->isChecked()) ? QString("-1.0") : numbers.sprintf("%f",CameraDofDepthEntry->text().toFloat()));
			sequence = sequence + ",\"radius\"," + numbers.sprintf("%f",CameraDofRadiusEntry->text().toFloat());
			sequence = sequence + ",\"mask\"," + CameraDofMaskSpin->text();
			break;
		case film :
			sequence = sequence + "\"grain\"," + numbers.sprintf("%f",CameraFilmGrainEntry->text().toFloat());
			break;
		case glitter :
			sequence = sequence + "\"radius\"," + numbers.sprintf("%f",CameraGlitterRadiusEntry->text().toFloat());
			sequence = sequence + ",\"intensity\"," + numbers.sprintf("%f",CameraGlitterIntensityEntry->text().toFloat());
			sequence = sequence + ",\"threshold\"," + numbers.sprintf("%f",CameraGlitterThresholdEntry->text().toFloat());
			break;
		case halo :
			sequence = sequence + "\"kr\"," + numbers.sprintf("%f",CameraHaloKrEntry->text().toFloat());
			sequence = sequence + ",\"km\"," + numbers.sprintf("%f",CameraHaloKmEntry->text().toFloat());
			sequence = sequence + ",\"shininess\"," + numbers.sprintf("%f",CameraHaloShEntry->text().toFloat());
	}	
	sequence = sequence + "])";	
	return(sequence);
}
/**
	* generates the source code for a lightflow default interface
	*/
QString Kmatedit::makeInterface()
{
	QString numbers;
	QString sequence = SceneObjEntry->text() + ".newInterface(\"default\",[\n";

	// first some flag vars
	sequence = sequence + "\t\"displacement\"," + ((InterfaceGeneralDisplacementCheck->isChecked()) ? "1" : "0") + "\n";
	sequence = sequence + "\t,\"shadows\"," + ((InterfaceGeneralShadowCheck->isChecked()) ? "1" : "0") + "\n";
	sequence = sequence + "\t,\"volumes\"," + ((InterfaceGeneralVolumeCheck->isChecked()) ? "1" : "0") + "\n";	
	
	if(InterfaceResourcesProcessorsCheck->isChecked())
	{
		sequence = sequence + "\t,\"processors\"," + InterfaceResourceProcessorsSpin->text() + "\n";
	}
	if(InterfaceResourceCachingCheck->isChecked())
	{
		sequence = sequence + "\t,\"caching\"," + InterfaceResourcesCellClustersSpin->text() + ",";
		sequence = sequence + InterfaceResourcesCellsperClusterSpin->text() + ",";
		sequence = sequence + numbers.sprintf("%f,%f\n",InterfaceResourcesMinRadiusEntry->text().toFloat(),\
		InterfaceResourcesMaxRadiusEntry->text().toFloat());
	}
	if(InterfaceResourcesTraceDepthCheck->isChecked())
	{
		sequence = sequence + "\t,\"trace-depth\"," + InterfaceGeneralTraceDepthSpin->text() + "\n";
	}
	// Radiosity Tab next...
	if(InterfaceRadiosityDepthCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-depth\"," + numbers.sprintf("%i,%i\n",InterfaceRadiosityDepthDiffuseSpin->value(),\
		InterfaceRadiosityDepthSpecularSpin->value());
	}
	if(InterfaceRadiositySamplesCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-samples\"," + InterfaceRadiositySamplesSpin->text() + "\n";
	}
	if(InterfaceRadiosityThresholdCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-threshold\"," + numbers.sprintf("%f\n",InterfaceRadiosityThresholdSpin->value()/100.0f);
	}
	if(InterfaceRadiositySamplingMethodCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-sampling\",\"" + InterfaceRadiositySamplingMethodCombo->currentText() + "\"\n";
	}
	if(InterfaceRadiosityReuseDistCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-reuse-distance\",";
		if(InterfaceRadiosityScreenCheck->isChecked())
		{
			sequence = sequence + numbers.sprintf("%f,%f,%f\n",InterfaceRadiosityScreenEntry->text().toFloat(),\
			InterfaceRadiosityReuseMaxDistEntry->text().toFloat(),InterfaceRadiosityReuseMaxDistEntry->text().toFloat());
		}
		else
		{
			sequence = sequence + numbers.sprintf("%f,%f\n",InterfaceRadiosityReuseMaxDistEntry->text().toFloat(),\
			InterfaceRadiosityReuseMaxDistEntry->text().toFloat());
		}
	}
	if(InterfaceRadiosityPatchEdgeCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-patch-edge\"," + numbers.sprintf("%f,%f\n",InterfaceRadiosityPatchEdgeMaxEntry->text().toFloat(),\
		InterfaceRadiosityPatchEdgeMinEntry->text().toFloat());
	}
	if(InterfaceRadiosityFileCheck->isChecked())
	{
		sequence = sequence + "\t,\"radiosity-file\",\"" + InterfaceRadiosityFileNameEntry->text() + "\",\"" + \
		InterfaceRadiosityFileModeCombo->currentText() + "\"\n";
	}	
	if(InterfaceLightningAccuracyCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-accuracy\"," + numbers.sprintf("%f\n",InterfaceLightningAccuracySpin->value()/100.0f);
	}
	if(InterfaceLightningCountCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-count\"," + InterfaceLightningCountSpin->text() + "\n";
	}
	if(InterfaceLightningDepthCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-depth\"," + InterfaceLightningDepthSpin->text() + "\n";
	}
	if(InterfaceLightningThresholdCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-threshold\"," + numbers.sprintf("%f\n",InterfaceLightningThresholdEntry->text().toFloat());
	}
	if(InterfaceLightningPatchEdgeCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-patch-edge\"," + numbers.sprintf("%f,%f\n",\
		InterfaceLightningThresholdMaxEntry->text().toFloat(),InterfaceLightningThresholdMinEntry->text().toFloat());
	}
	if(InterfaceLightningAngleCheck->isChecked())
	{
		sequence = sequence + "\t,\"lighting-angle\"," + numbers.sprintf("%f,%f\n",\
		InterfaceLightningAngleMaxEntry->text().toFloat(),InterfaceLightningAngleMinEntry->text().toFloat());
	}
	if(InterfaceLightningPhotonCountCheck->isChecked())
	{
		sequence = sequence + "\t,\"photon-count\"," + InterfaceLightningPhotonCountSpin->text() + "\n";
	}
	if(InterfaceLightningPhotonClusterCheck->isChecked())
	{
		sequence = sequence + "\t,\"photon-clustering-count\"," + InterfaceLightningPhotonClusterDiffuseSpin->text() + "," + \
		InterfaceLightningPhotonCausticSpin->text() + "\n";
	}
	// und abschlieen
	sequence = sequence + "\t])";
	return sequence;
}
/**
	* Add a new entry to the list
	*/
void Kmatedit::TransformListAdd()
{
	QListViewItem *sucessor = TransformationExecutionOrderListView->currentItem();
	QListViewItem *newitem = new QListViewItem(TransformationExecutionOrderListView);
	alterOpListItem(newitem);
	TransformationExecutionOrderListView->insertItem(newitem);
	newitem->moveItem(sucessor);
}
/**
	* alter the current entry
	*/
void Kmatedit::TransformListAlter()
{
	alterOpListItem(TransformationExecutionOrderListView->currentItem());
	TransformationExecutionOrderListView->triggerUpdate();
}
/**
	* append entry to the end of list
	*/
void Kmatedit::TransformListAppend()
{
	QListViewItem *temp,*newitem = new QListViewItem(TransformationExecutionOrderListView);
	
	alterOpListItem(newitem);
	TransformationExecutionOrderListView->insertItem(newitem);
	for(temp=newitem;temp->itemBelow() != 0; temp = temp->itemBelow()); // go to end of list
	newitem->moveItem(temp); // move to end of list
	TransformationExecutionOrderListView->triggerUpdate();	
}

/**
	* copy entry data to edit area and
 	* delete entry after
 	*/
void Kmatedit::TransformListCut()
{
	TransformListGet(TransformationExecutionOrderListView->currentItem());
	TransformListDelete();
}
/**
	* deletes the current entry
	*/
void Kmatedit::TransformListDelete()
{
	QListViewItem *todelete = TransformationExecutionOrderListView->currentItem();
	
	if(todelete != 0)
	{
		TransformationExecutionOrderListView->takeItem(todelete);
		// delete todelete;
		TransformationExecutionOrderListView->triggerUpdate();
	}
}

void Kmatedit::alterOpListItem(QListViewItem *item)
{
	item->setText(0,TransformationOperationTypeCombo->currentText()); // insert operation
	item->setText(1,TransformationXCoordEntry->text()); // insert X coord
	if(TransformationXforallCheck->isChecked()) // X value for all entries?
	{
		item->setText(2,TransformationXCoordEntry->text()); // insert y coord
		item->setText(3,TransformationXCoordEntry->text()); // insert z coord	
	}
	else
	{
		item->setText(2,TransformationYCoordEntry->text()); // insert y coord
		item->setText(3,TransformationZCoordEntry->text()); // insert z coord
	}	
}

/** feed data of current item to edit area */
void Kmatedit::TransformListGet(QListViewItem *item)
{
	int index;
	
	if(TransformationXforallCheck->isChecked())
	{
		TransformationXforallCheck->setChecked(false);
		TransformationYCoordEntry->setEnabled(true);
		TransformationZCoordEntry->setEnabled(true);
	}
	
	if(item->text(0).contains("move")) index = 0;
	if(item->text(0).contains("rotate")) index = 1;
	if(item->text(0).contains("scale")) index = 2;
	
	TransformationOperationTypeCombo->setCurrentItem(index);
	TransformationXCoordEntry->setText(item->text(1));
	TransformationYCoordEntry->setText(item->text(2));
	TransformationZCoordEntry->setText(item->text(3));
	
}
/** generates the python source code for a transformation stack  */
QString Kmatedit::makeTransformBlock()
{
	QString numbers,sequence,begintransform = SceneObjEntry->text() + ".transformBegin(transform().";
	QListViewItem *item = TransformationExecutionOrderListView->firstChild(); // take first item
	int operations = 0; // operations count numbers of opening block parts
	float temp;
	
	// list has to be evaluated top-down
	for(;item != 0;item = item->itemBelow(),operations++) // rush through list
	{
		if(item->text(0).contains("move"))
		{
				sequence = sequence + begintransform;
				sequence = sequence + "translation(vector3(" + item->text(1) + "," + item->text(2) + "," + item->text(3) + ")))\n";
		}	
		if(item->text(0).contains("scale"))
		{
				sequence = sequence + begintransform;
				sequence = sequence + "scaling(vector3(" + item->text(1) + "," + item->text(2) + "," + item->text(3) + ")))\n";
		}
		if(item->text(0).contains("rotate"))
		{
				operations--; // thrust me...
				if((temp = item->text(1).toFloat()) != 0.0f)
					{
						sequence = sequence + begintransform;
						sequence = sequence + "rotationAroundX(" + numbers.sprintf("%f))\n",toRadians(temp));
						operations++;
					}
				if((temp = item->text(2).toFloat()) != 0.0f)
					{
						sequence = sequence + begintransform;
						sequence = sequence + "rotationAroundY(" + numbers.sprintf("%f))\n",toRadians(temp));
						operations++;
					}
				if((temp = item->text(3).toFloat()) != 0.0f)
					{
						sequence = sequence + begintransform;
						sequence = sequence + "rotationAroundZ(" + numbers.sprintf("%f))\n",toRadians(temp));
						operations++;
					}
			}
		
	} // end for loop
	
	if(operations > 0)
	{
		sequence = sequence + "\n#inner level of block, edit here...\n\n";
		QString endofblockseq = SceneObjEntry->text() + ".transformEnd()\n";	
		for(;operations > 0;operations--) sequence = sequence + endofblockseq;
	}
	return sequence;
}
